"""
:authors: Runner
:license: Apache License, Version 2.0, see LICENSE file

:copyright: (c) 2021 Runner
"""

from .TZ1 import *
from .TZ2 import *
from .TZ3 import *
from .TZ4 import *
from .TZ5 import *
from .TZ6 import *
from .TZ7 import *
from .TZ8 import *

__author__ = 'Runner'
__version__ = '1.0.1'
__email__ = 'levintsov.v.v@gmail.com'


#!/usr/bin/env python
# coding: utf-8

# #### Библиотеки

# In[13]:


#  Математика  #
import numpy as np
import pywt
from sympy import *
from numpy import transpose
from numpy import linalg as LA
from fractions import Fraction
from scipy.fft import fft, ifft
#  Время  #
import time
# График #
import matplotlib.pyplot as plt
from matplotlib import rcParams
plt.rcParams['font.size'] = 36
plt.rcParams['font.family'] = 'Times New Roman'
plt.rcParams['mathtext.fontset'] = 'cm'
#  Другое  #
import webbrowser
import random
import copy
import csv


# #### Функции

# In[2]:


def csv_reader(string,sep):
    with open(string,newline='') as csvfile:
        reader = csv.reader(csvfile,delimiter=sep)
        lst=[]
        for row in reader:
            lst.append(list(row))
    return lst


# #### Основной код

# In[59]:


print('Доступные для тестирования файлы: \n 1) Test_sin.csv \n 2) sin_peak.csv \n 3) Line.csv \n 4) 2_sin_5_4.5.csv \n 5) RC_F12_11_2017_T12_11_2021.csv')
string = str(input('Введите название файла, который вы хотите протестировать: '))
sep = str(input('Укажитель разделитель (, или ;): '))
m=csv_reader(string,sep)
for i in range(len(m)):
    m[i][0]=float(m[i][0])
    m[i][1]=float(m[i][1])
    print(f'Значение {i+1}: [{m[i][0]},{m[i][1]}]')


# In[60]:


x = [m[i][0] for i in range(len(m))]
y = [m[i][1] for i in range(len(m))]
fi = list(fft(np.array(y)))
fi2 = list(ifft(fi))

coef_sin = abs((max(fi)/(sum(fi)-max(fi))).real)
print(f'Коэффициент несинусоидальности = {coef_sin}.')

disp = sum([(fi[i]-fi2[i])**2 for i in range(len(fi))]).real
print(f'Дисперсия = {disp}.')

rcParams['figure.figsize'] = (40, 30)
rcParams['figure.dpi'] = 400

plt.subplot(3, 1, 1)

plt.plot(x,y,color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Изначальный сигнал. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)

plt.subplot(3, 1, 2)

plt.plot(x,fi,color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Спектрограмма. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()

plt.subplot(3, 1, 3)

plt.plot(x,fi2,color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Восстановленная функция. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()


# In[42]:


def wav_sinus_coeff(coeffs):
    a4, d4, d3, d2, d1 = coeffs
    result = sqrt(np.std(d4) + np.std(d3) + np.std(d2) + np.std(d1)) / np.std(a4)
    return result


# ##### Daubechies (db)

# In[61]:


x = [i[0] for i in m]
y = [i[1] for i in m]
coeffs = pywt.wavedec(y, 'db5', level=4)

rcParams['figure.figsize'] = (40, 30)
rcParams['figure.dpi'] = 400

# Уровень 1

plt.subplot(4, 1, 1)

plt.plot(coeffs[-4],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Daubechies Уровень 1. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)

# Уровень 2

plt.subplot(4, 1, 2)

plt.plot(coeffs[-3],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Daubechies Уровень 2. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()

# Уровень 3

plt.subplot(4, 1, 3)

plt.plot(coeffs[-2],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Daubechies Уровень 3. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()

# Уровень 4

plt.subplot(4, 1, 4)

plt.plot(coeffs[-1],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Daubechies Уровень 4. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()

print(f'Коэффициент несинусоидальности: {wav_sinus_coeff(coeffs)}.')


# ##### Haar (haar)

# In[62]:


x = [i[0] for i in m]
y = [i[1] for i in m]
coeffs = pywt.wavedec(y, 'haar', level=4)

rcParams['figure.figsize'] = (40, 30)
rcParams['figure.dpi'] = 400

plt.subplot(4, 1, 1)

plt.plot(coeffs[-4],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Haar Уровень 1. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)



plt.subplot(4, 1, 2)

plt.plot(coeffs[-3],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Haar Уровень 2. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()



plt.subplot(4, 1, 3)

plt.plot(coeffs[-2],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Haar Уровень 3. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()



plt.subplot(4, 1, 4)

plt.plot(coeffs[-1],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Haar Уровень 4. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
plt.show()

print(f'Коэффициент несинусоидальности: {wav_sinus_coeff(coeffs)}.')


# ##### Mexican hat (mexh)

# In[63]:


x = [i[0] for i in m]
y = [i[1] for i in m]
wavelet = pywt.ContinuousWavelet('mexh')
coef, freqs = pywt.cwt(y,np.arange(1,30),wavelet)
endcoef = []
endcoef.append(coef[0])

plt.subplot(4, 1, 1)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Mexican hat Уровень 1. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)

plt.subplot(4, 1, 2)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Mexican hat Уровень 2. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

plt.subplot(4, 1, 3)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Mexican hat Уровень 3. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

plt.subplot(4, 1, 4)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Mexican hat Уровень 4. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

endcoef.append(sqrt(np.var(coef[-1])))
endcoef.reverse()
coefsin = sqrt(np.std(endcoef[1]) + np.std(endcoef[2]) + np.std(endcoef[3]) + np.std(endcoef[4])) / endcoef[0]
print(f'Коэффициент несинусоидальности: {coefsin}.')


# ##### Gaussian wavelets (gaus)

# In[64]:


x = [i[0] for i in m]
y = [i[1] for i in m]
wavelet = pywt.ContinuousWavelet('gaus5')
coef, freqs = pywt.cwt(y,np.arange(1,30),wavelet)
endcoef = []
endcoef.append(coef[0])

plt.subplot(4, 1, 1)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Gaussian wavelet Уровень 1. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)

plt.subplot(4, 1, 2)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Gaussian wavelet Уровень 2. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

plt.subplot(4, 1, 3)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Gaussian wavelet Уровень 3. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

plt.subplot(4, 1, 4)

plt.plot(coef[0],color='black',lw=2)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
plt.title(f'Gaussian wavelet Уровень 4. Файл {string}.',fontsize=28, loc = 'right')
plt.tick_params(labelsize = 40)
plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)
coef, freqs = pywt.cwt(coef[0],np.arange(1,30),wavelet)
endcoef.append(coef[0])

endcoef.append(sqrt(np.var(coef[-1])))
endcoef.reverse()
coefsin = sqrt(np.std(endcoef[1]) + np.std(endcoef[2]) + np.std(endcoef[3]) + np.std(endcoef[4])) / endcoef[0]
print(f'Коэффициент несинусоидальности: {coefsin}.')




#!/usr/bin/env python
# coding: utf-8

# №1. Транспонирование матрицы

# In[ ]:


# Вариант после исправления:
import numpy as np
row = int(input("Введите количество строк: "))  
column = int(input("Введите количество столбцов: "))  
 
matrix = []  
print("Заполните матрицу по одному элементу: ")  
 
for i in range(row):         
    a =[]  
    for j in range(column):      
        a.append(int(input()))  
    matrix.append(a)  
 
   
for i in range(row):  
    for j in range(column):  
        print(matrix[i][j], end = " ")  
    print()  
A1 = np.matrix(matrix)
Tr_A = A1.getT()
print('Исходная матрица: ', matrix)
print('Транспонированная матрица: ', Tr_A)


# Исходный вариант задания №1(до исправления):

# import numpy as np
# n, m = map(int, input("Введите кол-во строк и столбцов через пробел: ").split())
# A = list(map(int,input('Заполните матрицу через пробел: ').split()))
# A = [A[j * m : (j + 1) * m] for j in range(n)]
# A0 = A
# A1 = np.matrix(A)
# Tr_A = A1.getT()
# print('Исходная матрица: ', A0)
# print('Транспонированная матрица: ', Tr_A)

# №2. Матричное преобразование квадратных матриц 5 х 5 (Исправленый вариант по способу заполнения матрицы + вычисление норм матрицы и детерминанта)

# In[ ]:


#Вариант после исправления
import numpy as np
from numpy.linalg import norm
row = int(input("Введите количество строк первой матрицы: "))  
column = int(input("Введите количество столбцов первой матрицы: "))  
 
matrix = []  
print("Заполните матрицу по одному элементу: ")  
 
for i in range(row):         
    a =[]  
    for j in range(column):      
        a.append(int(input()))  
    matrix.append(a)  
 
 
for i in range(row):  
    for j in range(column):  
        print(matrix[i][j], end = " ")  
    print()  
A1 = np.matrix(matrix)

row2 = int(input("Введите количество строк второй матрицы: "))  
column2 = int(input("Введите количество столбцов второй матрицы: "))  
 
matrix2 = []  
print("Заполните матрицу по одному элементу: ")  
 
for i in range(row2):         
    a2 =[]  
    for j in range(column2):      
        a2.append(int(input()))  
    matrix2.append(a2)  
for i in range(row2):  
    for j in range(column2):  
        print(matrix2[i][j], end = " ")  
    print()  
B1 = np.matrix(matrix2)
znak = input('''Выберите '*'(умножение),'+'(сложение),'-'(вычитание), OPR(вычисление детерминанта),N2(сумма модулей элементов столбца): ''')
if znak == '*':
    if column != row2:
        print('Количество столбцов первой матрицы не равно количеству строк второй матрицы! Операция невозможа.')
    else:
        print('Результат умножения матриц:')
        print(A1*B1)
elif znak == '+':
    print('Результат сложения матриц:')
    print(A1+B1)
elif znak == '-':
    print('Результат вычитания матриц:')
    print(A1-B1) 
elif znak == 'OPR':
    print('Детерминант первой матрицы: ', np.linalg.det(A1))
    print('Детерминант второй матрицы: ', np.linalg.det(B1))
elif znak == 'N2':
    #Для первой матрицы
    if column and column2 == 2:
        fr2 = int(sum(A1[:,0]))
        se2 = int(sum(A1[:,1]))
        fr2_B = int(sum(B1[:,0]))
        se2_B = int(sum(B1[:,1]))
        print(max(fr2,se2))
        print(max(fr2_B,se2_B))
    elif column and column2 == 3:
        fr3 = int(sum(A1[:,0]))
        se3 = int(sum(A1[:,1]))
        th3 = int(sum(A1[:,2]))
        fr3_B = int(sum(B1[:,0]))
        se3_B = int(sum(B1[:,1]))
        th3_B = int(sum(B1[:,2]))
        print(max(fr3,se3,th3))
        print(max(fr3_B,se3_B,th3_B))
    elif column and column2 == 4:
        fr4 = int(sum(A1[:,0]))
        se4 = int(sum(A1[:,1]))
        th4 = int(sum(A1[:,2]))
        fo4 = int(sum(A1[:,3]))
        fr4_B = int(sum(B1[:,0]))
        se4_B = int(sum(B1[:,1]))
        th4_B = int(sum(B1[:,2]))
        fo4_B = int(sum(B1[:,3]))
        print(max(fr4,se4,th4,fo4))
        print(max(fr4_B,se4_B,th4_B,fo4_B))
    elif column and column2 == 5:
        fr5 = int(sum(A1[:,0]))
        se5 = int(sum(A1[:,1]))
        th5 = int(sum(A1[:,2]))
        fo5 = int(sum(A1[:,3]))
        fi5 = int(sum(A1[:,4]))
        fr5_B = int(sum(B1[:,0]))
        se5_B = int(sum(B1[:,1]))
        th5_B = int(sum(B1[:,2]))
        fo5_B = int(sum(B1[:,3]))
        fi5_B = int(sum(B1[:,4]))
        print(max(fr5,se5,th5,fo5,fi5))
        print(max(fr5_B,se5_B,th5_B,fo5_B,fi5_B))
  


# # Исходный вариант задания №2(до исправления):
# import numpy as np
# n, m = map(int, input("Введите кол-во строк и столбцов для первой (А) матрицы  через пробел: ").split())
# znak = input('''Выберите '*'(умножение),'+'(сложение),'-'(вычитание): ''')
# A = list(map(int,input('Заполните матрицу через пробел: ').split()))
# A = [A[j * m : (j + 1) * m] for j in range(n)]
# A1 = np.matrix(A)
# n1,m1 = map(int, input("Введите кол-во строк и столбцов для второй (В) матрицы  через пробел: ").split())
# B = list(map(int,input('Заполните матрицу B через пробел: ').split()))
# B = [B[j * m1 : (j + 1) * m1] for j in range(n1)]
# B1 = np.matrix(B)
# if znak == '*':
#     if m != n1:
#         print('Количество столбцов первой матрицы не равно количеству строк второй матрицы! Операция невозможа.')
#     else:
#         print('Результат умножения матриц:')
#         print(A1*B1)
# elif znak == '+':
#     print('Результат сложения матриц:')
#     print(A1+B1)
# elif znak == '-':
#     print('Результат вычитания матриц:')
#     print(A1-B1)
# 



#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy as np
import copy
import random
from math import exp
import matplotlib.pyplot as plt
Countries = {}
print("введите Вершины")
n = int(input())

def gs(way):
    sum=0
    for i in range(len(way)-1):
        sum+=Countries[int(str(way[i])+str(way[i+1]))]
    return sum
def Generatorrex():  # Авто генерация путей
    Well = []
    Well1 = []
    for i in range(1, n + 1):
        for x in range(1, n + 1):
            Countries[int(str(i) + str(x))] = int(random.randint(1, 20))
            if x == i:
                Countries[int(str(i) + str(x))] = "a"
            q = 0
            if q == 1:
                Countries[int(str(i) + str(x))] = "a"

            Well.append(Countries[int(str(i) + str(x))])
        Well1.append(Well)
        Well = []
    Well = []
    Well1 = []
    for i in range(1, n + 1):
        for x in range(1, n + 1):
            Countries[int(str(i) + str(x))] = Countries[int(str(x) + str(i))]
            Well.append(Countries[int(str(i) + str(x))])
        Well1.append(Well)
        Well = []



def waydown(way):
    counter=[]
    S=way[1:-1]
    S1=random.sample(S,2)
    for i in range(len(S)):
        if S[i]==S1[0]:
            counter.append(i)
        if S[i]==S1[1]:
            counter.append(i)

    timing = S[counter[0]]
    way[counter[0]+1]=S[counter[1]]
    way[counter[1]+1]=timing
    return way
def rnd():
    way=[]
    for i in range(1,n+1):
        way.append(i)
    random.shuffle(way)
    return way
Generatorrex()
x=rnd()
a=0.91
Tempreture=1000
Ll1=gs(x)
Counterstrike=0
Counterstrike2=[]

while Tempreture>0.01:
    Counterstrike+=1
    way3=waydown(x)
    Ll2=gs(way3)
    Difference=Ll2-Ll1
    Notcons=100*exp(-(Difference)/Tempreture)

    if Difference<0:
        Ll1=Ll2
        Tempreture=Tempreture*0.85
        #Counterstrike2.append(Ll1)
    elif Notcons > random.randint(1,100):
        Ll1 = Ll2
        Tempreture = Tempreture * 0.85
        #Counterstrike2.append(Ll1)
    else:
        Tempreture=Tempreture*0.85
    Counterstrike2.append(Ll1)

plt.plot(range(1,Counterstrike+1),Counterstrike2)
plt.show()




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy as np
from scipy.signal import argrelextrema
import matplotlib.pyplot as plt
import csv
import copy
import random
y=np.array([])
with open("cure.csv",encoding='utf-8-sig') as f:
    reader = csv.reader(f,delimiter=";")
    for row in reader:
        y=np.append(y,float(row[0]))
x = np.array(range(1,854))

PointsINMIn = argrelextrema(y, np.less_equal,order=20)[0]
PointsINMax = argrelextrema(y, np.greater_equal,order=20)[0]
Yy=copy.deepcopy(y)
Pairs=[]
plt.figure(figsize=(18,10))
plt.plot(x,y,c="red")
plt.title("Локальные экстремчики")
plt.scatter(PointsINMIn,y[PointsINMIn], c = "black")
plt.scatter(PointsINMax,y[PointsINMax],c="blue")
plt.show()
if (PointsINMIn[0]<PointsINMax[0]):
    a=1
else:
    a=0

if (PointsINMIn[-1]>PointsINMax[-1]):
    q=1
else:
    q=0
PointsINMInCheck=list(PointsINMIn)
PointsINMaxCheck=list(PointsINMax)

n=0

def greenPointsForCheck():
    if a==1:
        if PointsINMaxCheck[n+1]<PointsINMInCheck[n+1]:
            PointsINMaxCheck.pop(n+1)
            greenPointsForCheck()
    if a==0:
        if PointsINMaxCheck[n+1]<PointsINMInCheck[n]:
            PointsINMaxCheck.pop(n+1)
            greenPointsForCheck()


def appr_kx(x,y):
    from scipy.linalg import solve
    import matplotlib.pyplot as plt
    kx = solve([[sum([i ** 2 for i in x]), sum(x)], [sum(x), len(x)]],
               [[sum([y[i] * x[i] for i in range(len(x))])], [sum(y)]])
    x1 = []
    y1 = []
    for i in range(int(min(x)), int(max(x)) + 1):
        x1.append(i)
        y1.append(kx[0][0] * i + kx[1][0])
    print(f'y = {round(kx[0][0])}x+{round(kx[1][0])}')
    print()
    for ind, el in enumerate(x):
        print(f'x:{el} y:{y[ind]} fi:{round(kx[0][0] * el + kx[1][0])}')
    print()
    print(f'Дисперсия {sum([(y[ind] - kx[0][0] * el + kx[1][0]) ** 2 for ind, el in enumerate(x)])}')
    plt.plot(x1, y1)
    plt.scatter(x, y)
    return x1,y1

def appr_ax(x,y):
    from scipy.linalg import solve
    import matplotlib.pyplot as plt
    kx = solve([[sum([i ** 4 for i in x]), sum([i ** 3 for i in x]), sum([i ** 2 for i in x])],
                [sum([i ** 3 for i in x]), sum([i ** 2 for i in x]), sum([i ** 1 for i in x])],
                [sum([i ** 2 for i in x]), sum([i ** 1 for i in x]), len(x)]],
               [[sum([(x[i] ** 2) * y[i] for i in range(len(x))])],
                [sum([x[i] * y[i] for i in range(len(x))])],
                [sum([y[i] for i in range(len(x))])]])

    x1 = []
    y1 = []
    for i in range(int(min(x)) * 10, (int(max(x)) + 1) * 10):
        x1.append(i / 10)
        y1.append(kx[0][0] * (i / 10) ** 2 + kx[1][0] * i / 10 + kx[2][0])
    print(f'y = {round(kx[0][0])}x**2+{round(kx[1][0])}x + {round(kx[2][0])}')
    print()
    for ind, el in enumerate(x):
        print(f'x:{el} y:{y[ind]} fi:{kx[0][0] * (el) ** 2 + kx[1][0] * el + kx[2][0]}')
    print()
    print(
        f'Дисперсия {sum([(y[ind] - kx[0][0] * (x[ind]) ** 2 + kx[1][0] * x[ind] + kx[2][0]) ** 2 for ind, el in enumerate(x)])}')
    plt.plot(x1, y1)
    plt.scatter(x, y)
    return x1,y1

def RedPointsForCheck():
    if a==1:
        if PointsINMInCheck[n+1]<PointsINMaxCheck[n]:
            PointsINMInCheck.pop(n+1)
            RedPointsForCheck()
    if a==0:
        if PointsINMInCheck[n+1]<PointsINMaxCheck[n+1]:
            PointsINMInCheck.pop(n+1)
            RedPointsForCheck()
while (n<=len(PointsINMaxCheck)-2):
    greenPointsForCheck()
    RedPointsForCheck()
    greenPointsForCheck()
    RedPointsForCheck()
    n=n+1
n=0

#while (n<=len(PointsINMInCheck)-2):
 #   RedPointsForCheck()
#    n=n+1
#n=0



print(PointsINMInCheck)
print(PointsINMaxCheck)
n=0
while n<len(PointsINMInCheck)-2:
    Pairs.append([PointsINMInCheck[n],PointsINMaxCheck[n]])
    Pairs.append([PointsINMaxCheck[n],PointsINMInCheck[n+1]])
    n=n+1
print(Pairs)
MainPoimt=[]
a=0
while a<len(Pairs):
    XXX=round((Pairs[a][0]+Pairs[a][1])/2)
    MainPoimt.append(XXX)
    a=a+1
print(MainPoimt)
plt.figure(figsize=(18, 10))
plt.plot(x,y)
plt.title("Выборки для апроксимации")
plt.scatter(PointsINMInCheck,y[PointsINMInCheck],c="r")
plt.scatter(PointsINMaxCheck,y[PointsINMaxCheck],c="g")

for a in MainPoimt:
    plt.axvline(a,0,100,c="r")
print(y[30]-30)
plt.show()
ThangS=[]
ThangS2=[]
Expa1=[]
Expa2=[]
Expa3=[]
Expa4=[]
Dispp=0
for n in range(len(MainPoimt) - 1):

    Ymeans = []
    Xmeans = []
    cout = random.randint(0, 1)
    III = MainPoimt[n]
    while (III <= MainPoimt[n + 1]):
        Ymeans.append(y[III])
        Xmeans.append(x[III])
        III += 1
    x0 = MainPoimt[n]
    x1 = MainPoimt[n + 1]
    y0diff = []
    y1diff = []


    will2 = appr_kx(Xmeans,Ymeans)
    will1 = appr_ax(Xmeans,Ymeans)
    Ya5 = []
    Ya6 = []
    for ii in range(len(Ymeans)):
        y0diff.append((Ymeans[ii] - will2[1][ii]) ** 2)
        y1diff.append((Ymeans[ii] - will1[1][ii]) ** 2)
    Ya5 = appr_ax(Xmeans,Ymeans)
    Ya6 = appr_kx(Xmeans,Ymeans)
    if y0diff[-1] <= y1diff[-1]:
        will = appr_ax(Xmeans,Ymeans)
        Dispp = y0diff[-1] + Dispp
    if y0diff[-1] > y1diff[-1]:
        will = appr_kx(Xmeans,Ymeans)
        Dispp = y1diff[-1] + Dispp
    ThangS += will[0]
    ThangS2 += will[1]
    Expa1 += Ya5[0]
    Expa2 += Ya5[1]
    Expa3 += Ya6[0]
    Expa4 += Ya6[1]
Hah1=round(sum(y0diff),3)
Hah2=round(sum(y1diff),3)
Dispp=round(Dispp,3)
plt.subplot(2,3,1)
plt.show()
plt.plot(Expa1,Expa2,c="red")
plt.text(900, 75, Hah2, fontsize=15)
plt.text(800, 75,"Disp=", fontsize=15)
plt.title("Квадратичная кусочная аппроксимация")
plt.show()
#plt.subplot(2,3,1)
plt.text(1000, 75, Hah1, fontsize=15)
plt.text(800, 75,"Disp=", fontsize=15)
plt.plot(Expa3,Expa4,c="red")
plt.title("Линейная кусочная аппроксимация")
plt.show()
#plt.subplot(2,3,1)
plt.text(1000, 75, Dispp, fontsize=15)
plt.text(800, 75,"Disp=", fontsize=15)
plt.plot(ThangS,ThangS2, c="red")
plt.title("Кусочная аппроксимация обоими методами, зависимыми от дисперсии")
plt.show()




#!/usr/bin/env python
# coding: utf-8

# In[1]:


import copy

import random
import time
import matplotlib.pyplot as plt

print("введите Вершины")
n = int(input())




Q = 4  # константа 
a = 1  # константа 
b = 4  # константа 
Mnojest = {1}
Inters = []
Inters.append(1)
Countries = {}  # Множество городов
MAtrizaPheromonov = {}  # матрица феромончиков
Xsheeesh = []  # x координаты городов
Ysheeesh = []  # y координаты городов
Xsheeesh.append(1)  # Начинаем с 0, а не с 1
Ysheeesh.append(1)  # Начинаем с 0, а не с 1
for i in range(2, n + 1):
    Mnojest.add(i)






for i in range(1, n + 1):
    Xsheeesh.append(random.randint(200, 1500))
    Ysheeesh.append(random.randint(200, 900))


def GeneratorRex():  # Авто генерация путей
    Wow = []
    Wow1 = []
    for i in range(1, n + 1):
        for x in range(1, n + 1):
            Countries[int(str(i) + str(x))] = int(random.randint(100, 1710))
            if x == i:
                Countries[int(str(i) + str(x))] = "a"
            q = 0
            if q == 1:
                Countries[int(str(i) + str(x))] = "a"
            Wow.append(Countries[int(str(i) + str(x))])
        Wow1.append(Wow)
        Wow = []
    for i in Wow1:
        print(i)
        
def MatrizaPher():
    Wow = []
    Wow1 = []
    for i in range(1, n + 1):
        for x in range(1, n + 1):
            MAtrizaPheromonov[int(str(i) + str(x))] = 3
            if x == i:
                MAtrizaPheromonov[int(str(i) + str(x))] = "b"
            q = random.randint(1, 5)
            if Countries[int(str(i) + str(x))] == "a":
                MAtrizaPheromonov[int(str(i) + str(x))] = "b"
            Wow.append(MAtrizaPheromonov[int(str(i) + str(x))])
        Wow1.append(Wow)
        Wow = []
    for i in Wow1:
        print(i)

def GeneratorRex2():
    Wow = []
    Wow1 = []

    for i in range(1, n + 1):
        for x in range(1, n + 1):
            print(i, x, "введите:")
            Countries[int(str(i) + str(x))] = input()
            if x == i:
                Countries[int(str(i) + str(x))] = "a"
            Wow.append(Countries[int(str(i) + str(x))])
        Wow1.append(Wow)
        Wow = []
    for i in Wow1:
        print(i)


print("Авто или ручками (1 или 2)?")
Turn = int(input())
if Turn == 1:
    GeneratorRex()
if Turn == 2:
    GeneratorRex2()
print()
print("МАТРИЦА ФЕРОМОНчиков")
print()
MatrizaPher()


class DarkArmy():
    Piii = 1
    xlocation = Xsheeesh[Piii]
    ylocation = Ysheeesh[Piii]
    xfutlocation = 0
    yfuturelocation = 0
    Saving = copy.deepcopy(Mnojest)
    FutureCountries = []
    FutureCu = 0
    FutureCountriesP = []
    PastCountries = []
    Q = 200
    a = 1
    b = 1
    Y = 4
    Ready = 0
    countoffailures = 2

    def __init__(self, num):
        self.Piii = num
        self.xlocation = Xsheeesh[num]
        self.ylocation = Ysheeesh[num]

        self.Mnojest = {num}
        self.PastCountries = [num]
        print(self.PastCountries)

    def choosetown(self):
        if self.Ready == 0:
            self.FutureCountries = []
            self.FutureCountriesP = []
            for i in range(1, n + 1):
                if type(Countries[int(str(self.Piii) + str(i))]) is int and i not in self.PastCountries:
                    self.FutureCountries.append(int(i))
            if not self.FutureCountries and self.Saving != self.Mnojest:
                self.FutureCu = self.PastCountries[-(self.countoffailures)]
                self.xfutlocation = Xsheeesh[int(self.FutureCu)]
                self.yfuturelocation = Ysheeesh[int(self.FutureCu)]
                self.PastCountries.append(self.FutureCu)
                self.Mnojest.add(self.FutureCu)
                self.countoffailures += 1
            else:
                for i in self.FutureCountries:
                    self.FutureCountriesP.append(((Q / Countries[int(str(self.Piii) + str(i))]) ** 4) * (
                                (MAtrizaPheromonov[int(str(self.Piii) + str(i))]) ** 2))
                self.FutureCu = random.choices(self.FutureCountries, weights=self.FutureCountriesP, k=1)
                self.xfutlocation = Xsheeesh[int(self.FutureCu[0])]
                self.yfuturelocation = Ysheeesh[int(self.FutureCu[0])]
                self.PastCountries.append(self.FutureCu[0])
                self.countoffailures = 2
                self.Mnojest.add(self.FutureCu[0])
            # print(self.PastCountries)

    def NextStep(self):
        if self.Ready == 0:
            self.xlocation = self.xfutlocation
            self.ylocation = self.yfuturelocation

    def check(self):

        if Mnojest == self.Mnojest:
            self.Ready = 1

        return self.Ready


# for i in range(1,n+1):
# KK=random.Turns([Ant1,Ant2,Ant3],[10,10,10])
# Anty[i]=DarkArmy(ANttype=KK[0],num=i)


Nya1 = []
Nya2 = []
Lelwd3 = 0
Counter = 1
hmph = []
MinID = 0
for i in range(80):

    Anty = []
    for i in range(n + 1):
        Anty.append(1)

    for i in range(1, n + 1):
        Anty[i] = DarkArmy(num=i)
    while True:
        for i in range(1, n + 1):
            Anty[i].choosetown()

            time.sleep(0.0000002)
        for i in range(1, n + 1):
            Anty[i].NextStep()
        AAA = 0
        for i in range(1, n + 1):
            AAA += Anty[i].check()

        if AAA == n:
            break
        AAA = 0
    for i in range(1, n + 1):
        for YYY in range(1, n):

            Qar = copy.copy(Anty[i].PastCountries)

            Qar.insert(0, 1)

            if type(MAtrizaPheromonov[int(str(Qar[YYY]) + str(Qar[YYY + 1]))]) is int or float:
                MAtrizaPheromonov[int(str(Qar[YYY]) + str(Qar[YYY + 1]))] = MAtrizaPheromonov[int(str(Qar[YYY]) + str(
                    Qar[YYY + 1]))] * 0.64 + Q / Countries[int(str(Qar[YYY]) + str(Qar[YYY + 1]))]

    Nya1 = copy.copy(Anty[1].PastCountries)
    hmph.append(Nya1)
    Lelwd3 = 0
    for i in range(len(Nya1) - 1):
        Lelwd3 += Countries[int(str(Nya1[i]) + str(Nya1[i + 1]))]
        print(Countries[int(str(Nya1[i]) + str(Nya1[i + 1]))])
    Nya2.append(Lelwd3)

    Counter += 1
Checking = Nya2[0]
for i in range(len(Nya2)):
    if Checking > Nya2[i]:
        Checking = Nya2[i]
        MinID = i

print("самый короткий de way", hmph[MinID], Checking)
print("константы a=4,b=1. Q=300. Относительно оптимальный выбор для дорог сопоставимых с данными экрана (1920*1080)")
plt.plot(range(Counter - 1), Nya2)

plt.show()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# #### Библиотеки

# In[1]:


#  Математика  #
import numpy as np
from sympy import *
from numpy import transpose
from numpy import linalg as LA
from fractions import Fraction
#  Время  #
import time
# График #
import matplotlib.pyplot as plt
from matplotlib import rcParams
plt.rcParams['font.size'] = 36
plt.rcParams['font.family'] = 'Times New Roman'
plt.rcParams['mathtext.fontset'] = 'cm'
#  Другое  #
import random
import copy
import csv


# #### Функции

# In[2]:


# Функция проверки числа

def number(str_input, str_error, str_error2, type_num): # str_input - строка выводимая пользователю при вводе 
                                                        # str_error - ошибка: число не является числом ('строка')
                                                        # str_error2 - число не соответсвует указаным требованиям
                                                        # type_num - все допустимые типы чисел
    print(str_input)
    num = input()
    if 'i' in num:
        num = itojnum(num)
    num.replace(" ", "")
    try:
        check = complex(num) # Проверка: является ли числом (комплексное можем взять от любого числа)
    except ValueError:
        print(str_error)
        return number(str_input, str_error, str_error2, type_num)
    
    if (complex in type_num) and check.imag != 0: # Проверки для комплексных чисел
        return jtoinum(num)
    elif (complex in type_num) and check.imag == 0:
        if (int in type_num):
            if check.real == round(check.real):
                return str(int(check.real))
        if (float in type_num):
            if check.real != round(check.real):
                return str(float(check.real))
        else:
            print(str_error2)
            return number(str_input, str_error, str_error2, type_num)

    elif (float in type_num): # Проверки для вещественных чисел
        if check.imag != 0:
            print(str_error2)
            return number(str_input, str_error, str_error2, type_num)
        if (int in type_num):
            if check.real == round(check.real):
                return str(int(check.real))
        else:
            return str(float(check.real))

    else: # Проверки для целых чисел
        if check.imag != 0:
            print(str_error2)
            return number(str_input, str_error, str_error2, type_num)
        elif check.real != round(check.real):
            print(str_error2)
            return number(str_input, str_error, str_error2, type_num)
        return str(int(check.real))
    
# Функция генерации рандомных чисел для CSV-файла и python.
    
def random_numbers(row,minim,maxi):
    complex_numb=[]
    for i in range(row**3):
        floatnump=random.randint(1,6)
        numb_of_list=random.randint(1,2)
        if numb_of_list==1:
            a=random.randint(minim,maxi)
        else:
            a=round(random.uniform(minim,maxi),floatnump)
        numb_of_list=random.randint(1,2)
        if numb_of_list==1:
            b=random.randint(minim,maxi)
        else:
            b=round(random.uniform(minim,maxi),floatnump)
        complex_numb.append(complex(a,b))
    
    result=[0]*row
    for i in range(row):
        floatnump=random.randint(1,6)
        numb_of_list=random.randint(1,3)
        if numb_of_list==1:
            result[i]=str(random.randint(minim,maxi))
        if numb_of_list==2:
            result[i]=str(round(random.uniform(minim,maxi),floatnump))
        if numb_of_list==3:
            result[i]=str(random.choice(complex_numb))
    
    return result

# Функция ввода матрицы через клавиатуру
    
def default_matrix(): # N - кол-во строк, M - кол-во столбцов
    try:
        rowcol = list(map(int,input('Введите количество строк и столбцов: ').split()))
        N = rowcol[0]
        M = rowcol[1]
        if len(rowcol) > 2:
            print('Введено слишком много значений. Попробуйте ещё раз.')
            return default_matrix()
    except ValueError:
        print('Введено не целое значение строки и/или столбца. Попробуйте ещё раз.')
        return default_matrix()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return default_matrix()
    if N == 0 or M == 0:
        print('Введено нулевое значение! Количество строк и столбцов должно быть минимум 1!!')
        return default_matrix()
    mtx = [[0] * M for i in range(N)]
    for n in range(N):
        for m in range(M):
            mtx[n][m] = number(f'Введите значение для элемента матрицы a[{n + 1}][{m + 1}]: ',
                              'Введено неверное выражение. Попробуйте ещё раз', 
                              'Введено число в неверном формате. Попробуйте ещё раз.',
                              [complex, float, int])
            
    for n in range(len(mtx)):
        #mtx[n].append('|')
        mtx[n].append(number(f'Введите значение для свободного члена {n + 1} строки: ',
                              'Введено неверное выражение. Попробуйте ещё раз',
                              'Введено число в неверном формате. Попробуйте ещё раз.',
                              [complex, float, int]))
    return mtx

# Функция ввода матрицы через радомную генерацию python.

def python_generator():
    try:
        rowcol = list(map(int,input('Введите количество строк и столбцов (N M): ').split()))
        N = rowcol[0]
        M = rowcol[1]
        if len(rowcol) > 2:
            print('Введено слишком много значений. Попробуйте ещё раз.')
            return python_generator()
    except ValueError:
        print('Введено не целое значение строки и/или столбца. Попробуйте ещё раз.')
        return python_generator()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return python_generator()
    if N == 0 or M == 0:
        print('Введено нулевое значение! Количество строк и столбцов должно быть минимум 1!!')
        return python_generator()

    try:
        minmax = list(map(int,input('Введите минимальное и максимальное значене для элемента матрицы (также для мнимой части комплексного числа) (min max): ').split()))
        mini = minmax[0]
        maxi = minmax[1]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return python_generator()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return python_generator()
    if mini > maxi:
        print(f'Минимальное число не может быть больше максимального ({mini}!>{maxi})!!')
        return python_generator()
    
    result=[]
    for i in range(M):
        result.append(random_numbers(N,mini,maxi))
    
    for row in range(len(result)):
        #result[row].append('|')
        result[row].append(random_numbers(1,mini,maxi))
        result[row][-1]=str(result[row][-1][0])
        
    result=jtoi(result)
    result=del_bracket(result)
        
    return result

# Функция ввода матрицы через CSV-файл.

def csv_generator():
    try:
        rowcol = list(map(int,input('Введите количество строк и столбцов (N M): ').split()))
        N = rowcol[0]
        M = rowcol[1]
        if len(rowcol) > 2:
            print('Введено слишком много значений. Попробуйте ещё раз.')
            return csv_generator()
    except ValueError:
        print('Введено не целое значение строки и/или столбца. Попробуйте ещё раз.')
        return csv_generator()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return csv_generator()
    if N == 0 or M == 0:
        print('Введено нулевое значение! Количество строк и столбцов должно быть минимум 1!!')
        return csv_generator()

    try:
        minmax = list(map(int,input('Введите минимальное и максимальное значене для элемента матрицы (также для мнимой части комплексного числа) (min max): ').split()))
        mini = minmax[0]
        maxi = minmax[1]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return csv_generator()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return csv_generator()
    if mini > maxi:
        print(f'Минимальное число не может быть больше максимального ({mini}!>{maxi})!!')
        return csv_generator()
    
    result=[]
    for i in range(M):
        result.append(random_numbers(N,mini,maxi))
        
    for row in range(len(result)):
        #result[row].append('|')
        result[row].append(random_numbers(1,mini,maxi))
        result[row][-1]=str(result[row][-1][0])
        
    result=jtoi(result)
    result=del_bracket(result)
    
    with open('Answer_file.csv','w',newline='') as csvfile:
        writer=csv.writer(csvfile,delimiter=';')
        for row in result:
            writer.writerow(row)

    Matrix_in=[]
    with open('Answer_file.csv',newline='') as csvfile:
        reader = csv.reader(csvfile,delimiter=';')
        Matrix_in=[]
        for row in reader:
            Matrix_in.append(list(row))
    return Matrix_in

# Функция преобразования "i" в "j" для списка

def itoj(mtx):
    ans = []
    for i in range(len(mtx)):
        temp = []
        y = mtx[i]
        for j in y:
            temp.append(j.replace('i','j'))
        ans.append(temp)
    return ans

# Функция преобразования "j)" в "i" для списка.

def jtoi(mtx):
    ans = []
    for i in range(len(mtx)):
        temp = []
        y = mtx[i]
        for j in y:
            temp.append(j.replace('j)','i'))
        ans.append(temp)
    return ans

# Функция преобразования удаления левой скобки для списка.

def del_bracket(mtx):
    ans = []
    for i in range(len(mtx)):
        temp = []
        y = mtx[i]
        for j in y:
            temp.append(j.replace('(',''))
        ans.append(temp)
    return ans

# Функция преобразования "i" в "j" для строки.

def itojnum(st):
    ans = ''
    for i in st:
        ans += i.replace('i','j')
    return ans

# Функция преобразования "j" в "i" для строки.

def jtoinum(st):
    ans = ''
    for i in st:
        ans += i.replace('j','i')
    return ans

# Функция итерации матрицы всеми способами.
        
def iteration():
    print("Как вы хотите ввести матрицу:\n 1 - С кливаитуры\n 2 - Рандомная генерация в python\n 3 - CSV Файл")
    try:    
        choice = int(input('Вы ввели: '))
        choices_dict = {1: default_matrix, 2: python_generator , 3: csv_generator}
        mtx = choices_dict[choice]()
    except KeyError:
        print('Введено неверное значение ввода матрицы. Попробуйте ещё раз.')
        return iteration()
    except ValueError:
        print('Введено неверное значение ввода матрицы. Попробуйте ещё раз.')
        return iteration()
    return mtx

# Преобразование строковых значений в комплексные

def str_to_complex(mtx1):
    mtx = copy.deepcopy(mtx1)
    for row in mtx:
        for i in range(len(row)):
            row[i]=complex(itojnum(row[i]))
    return(mtx)

# Функция преобразует комплексные числа в другие типы чисел

def complex_to_num(st):
    if st.imag==0:
        if round(st.real)==st.real:
            return int(st.real)
        else:
            return float(st.real)
    else:
        return complex(st.real, st.imag)
    
# Создание из строковых чисел правильные дроби
    
def numbers_to_fractions(mtx):
    for row in range(len(mtx)):
        for col in range(len(mtx[row])):
            if 'i' in mtx[row][col]:
                return 'Функция не работает с комплексными числами'
            mtx[row][col]=Fraction(mtx[row][col])
    return mtx
    
# Нахождение определителя матрицы

def det_my_matrix(mtx):
    
    Lmtx=len(mtx)
    
    if Lmtx==1:
        return mtx[0][0]
    if Lmtx==2:
        return mtx[0][0]*mtx[1][1]-(mtx[0][1]*mtx[1][0])
    
    result=0
    for i in range(Lmtx):
        
        factor=1
        if i % 2:
            factor=-1
            
        mtx2=[]
        for row in range(Lmtx):
            mtx3=[]
            for col in range(Lmtx):
                if row!=0 and col!=i:
                    mtx3.append(mtx[row][col])
            if mtx3:
                mtx2.append(mtx3)
        
        result+=factor*mtx[0][i]*det_my_matrix(mtx2)
    return(result)

# Вычисление обратной матрицы

def inverse_matrix(mtx):
    Lmtx = len(mtx)
    mult = det_my_matrix(mtx)
    if mult == 0:
        return 'Матрица вырожденная'
    ans = [[0] * Lmtx for i in range(Lmtx)]
    for i in range(Lmtx):  
        for j in range(Lmtx):
            factor=1
            if (i+j) % 2:
                factor=-1
            mtx2 = []
            for i1 in range(Lmtx):
                if i1 != i:
                    mtx3 = []
                    for j1 in range(Lmtx):
                        if j1 != j:
                            mtx3.append(mtx[i1][j1])
                    mtx2.append(mtx3)
            ans[j][i] = factor * det_my_matrix(mtx2) / mult
    return ans

def diag(mtx1):
    mtx = copy.deepcopy(mtx1)
    for row in range(len(mtx)):
        for col in range(len(mtx[row])):
            if row==col:
                mtx[row]=list(np.array(mtx[row])/mtx[row][col])
    return mtx

# Вычисление матрицы коэффициентов

def coeff_mtx(mtx):
    mtx1 = []
    for i in range(len(mtx)):
        mtx1.append(mtx[i][:-1])
    return mtx1

# Вычисление вектора своюодных членов

def coeff_vect(mtx):
    mtx1 = []
    for i in range(len(mtx)):
        mtx1.append(mtx[i][-1])
    return mtx1

# Вычисление матодом простых итераций Якоби

def jacobi(arr,x,acc):
    arr1 = coeff_mtx(arr)
    vect = coeff_vect(arr)
    D = np.diag(arr1)
    R = arr1 - np.diagflat(D)
    x1 = [i for i in x]
    x = (vect - np.dot(R,x)) / D
    fin = abs(x1 - x)
    itr = 0
    while max(fin)>=acc:
        if itr >= 100:
            return 'Матрица расходится'
        itr += 1
        x1 = [i for i in x]
        x = (vect - np.dot(R,x)) / D
        fin = abs(x1 - x)
    return x

# Метод простых итераций Якоби

def jacobian_method(mtx):
    mtx1 = str_to_complex(mtx)
    coeff = coeff_mtx(mtx1)
    vect = coeff_vect(mtx1)
    n = len(mtx)
    print('Прямая матрица коэффициентов:')
    for i in range(n):
        print(coeff[i])
    rev = inverse_matrix(coeff)
    print('Обратная матрица коэффициентов:')
    for i in range(n):
        print(rev[i])
    print('Решение СЛАУ методом простых итераций Якоби:')
    mtx2 = np.array(mtx1)
    x = np.array([0 for i in range(n)])
    acc = 0.001
    sol = jacobi(mtx2, x, acc)
    print(sol)
    print('Число обусловленности Матрицы Коэффициентов A: ')
    conditional_jac = LA.cond(coeff)
    print(conditional_jac)
    return conditional_jac

#Вычилсение методом Гаусаа-Жордана

def GJ_method(mtx1):
    mtx = copy.deepcopy(mtx1)
    n = len(mtx)
    if det_my_matrix(mtx) == 0:
        return 'Вырожденная матрица. Нормально не считается этим методом'
    for itr in range(n):
        mtx[itr] = [mtx[itr][i] / mtx[itr][itr] for i in range(n + 1)]
        for col in range(n):
            if col != itr:
                mtx[col] = [mtx[col][i] - mtx[itr][i] * mtx[col][itr] for i in range(n + 1)]
    for row in mtx:
        for i in range(len(row)):
            row[i] = complex_to_num(row[i])
            if abs(row[i]) < 10 ** -10:
                row[i] = 0
    return coeff_vect(mtx)

# Метод Гаусаа-Жордана  
    
def jordan_method(mtx):
    mtx1 = str_to_complex(mtx)
    coeff = coeff_mtx(mtx1)
    vect = coeff_vect(mtx1)
    n = len(mtx)
    print('Прямая матрица коэффициентов:')
    for i in range(n):
        print(coeff[i])
    rev = inverse_matrix(coeff)
    print('Обратная матрица коэффициентов:')
    for i in range(n):
        print(rev[i])
    print('Решение СЛАУ методом Жордана-Гаусса:')
    sol = GJ_method(mtx1)
    print(sol)
    print('Число обусловленности Матрицы Коэффициентов A: ')
    conditional_gauss = LA.cond(coeff)
    print(conditional_gauss)
    return conditional_gauss

#Вычилсение методом Гаусаа-Жордана для правильных дробей

def GJ_method_2(mtx1):
    mtx = copy.deepcopy(mtx1)
    n = len(mtx)
    if det_my_matrix(mtx) == 0:
        return 'Вырожденная матрица. Нормально не считается этим методом'
    for itr in range(n):
        mtx[itr] = [mtx[itr][i] / mtx[itr][itr] for i in range(n + 1)]
        for col in range(n):
            if col != itr:
                mtx[col] = [mtx[col][i] - mtx[itr][i] * mtx[col][itr] for i in range(n + 1)]
    ans = coeff_vect(mtx)
    return ans

# Метод Гаусаа-Жордана для правильных дробей
    
def jordan_method_2(mtx):
    mtx1 = numbers_to_fractions(mtx)
    coeff = coeff_mtx(mtx1)
    vect = coeff_vect(mtx1)
    n = len(mtx)
    print('Прямая матрица коэффициентов:')
    for i in range(n):
        print(coeff[i])
    rev = inverse_matrix(coeff)
    print('Обратная матрица коэффициентов:')
    for i in range(n):
        print(rev[i])
    print('Решение СЛАУ методом Жордана-Гаусса для Дробей:')
    sol = GJ_method_2(mtx1)
    for i in range(len(sol)):
        print(f'Значение x[{i + 1}] = {sol[i]}')
    for i in range(len(coeff)):
        for j in range(len(coeff[i])):
            coeff[i][j] = float(coeff[i][j])
    conditional_gauss2 = LA.cond(coeff)
    print('Число обусловленности Матрицы Коэффициентов A: ')
    print(conditional_gauss2)
    return conditional_gauss2

# Основное тело программы

def main():
    matrix=iteration()
    print('Введённая матрица:')
    print(matrix)
    print('Матрица коэфициентов:(Будет выводиться М а если матрица вырожденная)')
    print(coeff_mtx(matrix))
    print('Вектор свободных значений:')
    print(coeff_vect(matrix))
    jac = jacobian_method(matrix)
    if jac > 100:
        return 'Программа завершилась.'
    gauss = jordan_method(matrix)
    if gauss > 100:
        return 'Программа завершилась.'
    for row in matrix:
        for col in row:
            if 'i' in col:
                print('Нельзя считать дроби с комплексными числами.')
                print('Хотите ли попробовать ввести матрицу ещё раз? \n 1 - да \n 2 - нет')
                try:    
                    choice = int(input())
                    ext = lambda: 'Программа завершилась.'
                    choices_dict = {1: main, 2: ext}
                    mtx = choices_dict[choice]()
                except KeyError:
                    print('Введено неверное значение для ответа на вопрос. Запущен повторный ввод матрицы')
                    return main()
                except ValueError:
                    print('Введено неверное значение для ответа на вопрос. Запущен повторный ввод матрицы')
                    return main()
                return mtx
    try:
        gauss2 = jordan_method_2(matrix)
    except np.linalg.LinAlgError as err:
        print('Ошибка вычислений. Введена вырожденная матрица для которой не считается число обусловленности')
        print('Хотите ли попробовать ввести матрицу ещё раз? \n 1 - да \n 2 - нет')
        try:    
            choice = int(input())
            ext = lambda: 'Программа завершилась.'
            choices_dict = {1: main, 2: ext}
            mtx = choices_dict[choice]()
        except KeyError:
            print('Введено неверное значение для ответа на вопрос. Запущен повторный ввод матрицы')
            return main()
        except ValueError:
            print('Введено неверное значение для ответа на вопрос. Запущен повторный ввод матрицы')
            return main()
        return mtx


# In[3]:


matrix=iteration()
print('Введённая матрица:')
print(matrix)
print('Матрица коэфициентов:(Будет выводиться М а если матрица вырожденная)')
print(coeff_mtx(matrix))
print('Вектор свободных значений:')
print(coeff_vect(matrix))


# #### Метод простых итераций Якоби

# In[4]:


jacobi_matrix=copy.deepcopy(matrix)
jacobian_method(jacobi_matrix)


# ### Прямой алгоритм Гаусса-Жордана

# In[5]:


Gauss_Jordan_matrix = copy.deepcopy(matrix)
jordan_method(Gauss_Jordan_matrix)


# ### Прямой алгоритм Гаусса Жордана, с вычислениями правильных дробей

# In[6]:


Gauss_Jordan_matrix_2 = copy.deepcopy(matrix)
jordan_method_2(Gauss_Jordan_matrix_2)


# In[7]:


main()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[6]:


from numpy import *
import pandas as pd
import sympy as sm
import matplotlib.pyplot as plt


# ## T 1

# In[8]:


def task1(f, left, right, num_of_dots, dx):
    
    # ==========Задали функции==============
    
    x, y = sm.symbols('x y')
    
    func = sm.parse_expr(f)
    
    func_d = sm.diff(func)
    
    
    # =====Формула рассчета производной=====
    
    func_d_hands = lambda x1, dx: (func.evalf(subs={x:x1 + dx}) - func.evalf(subs={x:x1 - dx}))/(2*dx)
    
    # =======Определение интервала==========
    left_b = eval(str(left))
    right_b = eval(str(right))
    num = int(eval(str(num_of_dots))//1)
                                              
    x1 = list(linspace(left_b, right_b, num = num))
    
                                              
    # ==========Подсчет значений============
                                              
    y_f = []
    y_df1 = []
    y_df2 = []
    helpi = [] 
    for i in x1:
        try:
            a = func.evalf(subs={x:i})
            b = func_d.evalf(subs={x:i})
            c = func_d_hands(i, dx)
            y_f.append(a)
            y_df1.append(b)
            y_df2.append(c)
        except:
            helpi.append(i)
    
    for h in helpi:
        x1.remove(h)
    
    # ========Создание ДатаФрейма============
    
    df = pd.DataFrame({'x': x1, "f(x)": y_f, "f'(x)s": y_df1, "f'(x)h": y_df2})
    
    # ==========Построение графика============
    fig, axes = plt.subplots(1, 3)

    
    axes[0].plot(df['x'],df['f(x)'])
    axes[1].plot(df['x'],df["f'(x)s"])
    axes[2].scatter(df['x'],df["f'(x)h"],marker ='.',linewidths=0.5)


    axes[0].set_title("f(x)",
                    loc = 'center',
                    pad = 10,
                    fontsize = 20)    

    axes[1].set_title("f'(x)s",
                    loc = 'center',     
                    pad = 10,
                    fontsize = 20)

    axes[2].set_title("f'(x)h",
                    loc = 'center',
                    pad = 10,
                    fontsize = 20)


    for ax in axes:
        ax.grid(axis = 'both')

    fig.suptitle('Графики',
                    fontsize = 20,
                    y = 1.1)

    fig.set_figheight(4)
    fig.set_figwidth(16)

    plt.show()
    
    print(df)

df = task1('cos(x)', -1, 1, 40, 0.1)


# # T 2

# In[17]:


def task2(f, left, right):
    
    # ==========Задали функции==============
    
    x, y = sm.symbols('x y')
    
    func = sm.parse_expr(f)
    
    func_d = sm.diff(func)
    
    left_b = eval(str(left))
    right_b = eval(str(right))
    
    
    
    # constconstconstconstconstconstconstconst
    
    min_step = 0.01
    
    left_tr = left
    der_l = func_d.evalf(subs={x:left_tr})
    
    right_tr = left_tr + min_step
    
    
    square = 0
    list_of_tr = []
    
    # constconstconstconstconstconstconstconst
    
    
    
    while not(right_tr >= right_b):
        
        der_r = func_d.evalf(subs={x:right_tr})
        
        if abs(der_l - der_r) >= 0.01:
            
            list_of_tr.append([left_tr,func.evalf(subs={x:left_tr}),right_tr,func.evalf(subs={x:right_tr})])
            
            left_tr = right_tr
            right_tr = left_tr + min_step
            der_l =func_d.evalf(subs={x:left_tr})
            
        else:
            right_tr += min_step
            if right_tr >= right_b:
                list_of_tr.append([left_tr,func.evalf(subs={x:left_tr}),right_tr,func.evalf(subs={x:right_tr})])
         
    
#     =========== Построение графиков =============
    x_spl = list(linspace(left_b, right_b, int((abs(left_b-right_b)*100)//1)))
    y = []
    helpi = [] 
    for i in x_spl:
        try:
            a = func.evalf(subs={x:i})
            y.append(a)
        except:
            helpi.append(i)
    
    for h in helpi:
        x_spl.remove(h)
    
    
    fig, ax = plt.subplots()
    
    ax.plot(x_spl, y)
    
    for s in list_of_tr:
        ax.plot([s[0],s[0],s[2],s[2]],[0,s[1], s[3],0])
        square += (abs((s[0]-s[2]))*(s[1]+s[3]))/2
    
    print('S = ', square)
                     
            
        
task2('1/x', 2,20) 


# In[4]:


def antiderivative(f, left, right):
    
    x, y = sm.symbols('x y')
    
    func = sm.parse_expr(f)
    
    left_b = eval(str(left))
    right_b = eval(str(right))
    
    num_of_dots= (abs(left_b-right_b)*100)//1
    
    linsp_of_dots = linspace(left_b, right_b, num = num_of_dots)

       
    func_d_hands = lambda x1: (func.evalf(subs={x:x1 + 0.0001}) - func.evalf(subs={x:x1 - 0.0001}))/(2*0.0001)
    
    x1 = []
    y1 = []
    for k,i in enumerate(linsp_of_dots[:-1]):
        
        
        tangent = lambda x1: func.evalf(subs={x: i})+ func_d_hands(i)*(x1*i)
        
        y1.append(tangent(linsp_of_dots[k+1]))
        x1.append(linsp_of_dots[k+1])    

    plt.scatter(x1,y1,marker ='.',linewidths=0.1)

    
antiderivative('sin(x)', -30, 30)


# In[5]:


antiderivative('x**2', -10, 10)


# In[2]:


def S(f, left, right):
    
    # ==========Задали функции==============
    
    x, y = sm.symbols('x y')
    
    func = sm.parse_expr(f)
    
    func_d = sm.diff(func)
    
    left_b = eval(str(left))
    right_b = eval(str(right))
    
    
    
    # constconstconstconstconstconstconstconst
    
    min_step = 0.001
    
    left_tr = left
    der_l = func_d.evalf(subs={x:left_tr})
    
    right_tr = left_tr + min_step
    
    
    square = 0
    list_of_tr = []
    
    # constconstconstconstconstconstconstconst
    
    
    
    while not(right_tr >= right_b):
        
        der_r = func_d.evalf(subs={x:right_tr})
        
        if abs(der_l - der_r) >= 0.001:
            
            list_of_tr.append([left_tr,func.evalf(subs={x:left_tr}),right_tr,func.evalf(subs={x:right_tr})])
            
            left_tr = right_tr
            right_tr = left_tr + min_step
            der_l =func_d.evalf(subs={x:left_tr})
            
        else:
            right_tr += min_step
            if right_tr >= right_b:
                list_of_tr.append([left_tr,func.evalf(subs={x:left_tr}),right_tr,func.evalf(subs={x:right_tr})])
         
    for s in list_of_tr:

        square += (abs((s[0]-s[2]))*(s[1]+s[3]))/2
    
    return eval(str(square))


# In[3]:


def antiderivative_V2(f, left, right):
        
    x, y = sm.symbols('x y')
    
    func = sm.parse_expr(f)
    step = 0.1
    left_b = eval(str(left))
    left_h = left_b + step 
    right_b = eval(str(right))

    x1 = []
    y1 = []
    
    y1.append(S(f, left_b, left_h))

    x1.append(left_h)
    
    while not(left_h >= right_b):
        left_h += step

        y1.append(y1[-1] + S(f,x1[-1], left_h))
        x1.append(left_h)
    
    plt.scatter(x1,y1,marker ='.',linewidths=0.5)
    


# In[5]:


get_ipython().run_cell_magic('time', '', "antiderivative_V2('x**2',-5, 5)")


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# #### Библиотеки

# In[2]:


#  Математика  #
import numpy as np
from sympy import *
from scipy.integrate import odeint
import pandas as pd
#  Время  #
import time
# График #
import matplotlib.pyplot as plt
from matplotlib import rcParams
plt.rcParams['font.size'] = 36
plt.rcParams['font.family'] = 'Times New Roman'
plt.rcParams['mathtext.fontset'] = 'cm'
#  Другое  #
import webbrowser
import random
import copy
import csv
import time


# #### Функции

# In[3]:


def coeff_vect(mtx):
    mtx1 = []
    for i in range(len(mtx)):
        mtx1.append(mtx[i][-1])
    return mtx1

def det_my_matrix(mtx):
    
    Lmtx=len(mtx)
    
    if Lmtx==1:
        return mtx[0][0]
    if Lmtx==2:
        return mtx[0][0]*mtx[1][1]-(mtx[0][1]*mtx[1][0])
    
    result=0
    for i in range(Lmtx):
        
        factor=1
        if i % 2:
            factor=-1
            
        mtx2=[]
        for row in range(Lmtx):
            mtx3=[]
            for col in range(Lmtx):
                if row!=0 and col!=i:
                    mtx3.append(mtx[row][col])
            if mtx3:
                mtx2.append(mtx3)
        
        result+=factor*mtx[0][i]*det_my_matrix(mtx2)
    return(result)

def GJ_method_2(mtx1):
    mtx = copy.deepcopy(mtx1)
    n = len(mtx)
    if det_my_matrix(mtx) == 0:
        return 'Вырожденная матрица. Нормально не считается этим методом'
    for itr in range(n):
        mtx[itr] = [mtx[itr][i] / mtx[itr][itr] for i in range(n + 1)]
        for col in range(n):
            if col != itr:
                mtx[col] = [mtx[col][i] - mtx[itr][i] * mtx[col][itr] for i in range(n + 1)]
    return coeff_vect(mtx)

def quadratic_function(X,Y):
    sumX4 = sum([X[i] * X[i] * X[i] * X[i] for i in range(len(X))])
    sumX3 = sum([X[i] * X[i] * X[i] for i in range(len(X))])
    sumX2 = sum([X[i] * X[i] for i in range(len(X))])
    sumXY = sum([X[i] * Y[i] for i in range(len(X))])
    sumX2Y = sum([X[i] * X[i] * Y[i] for i in range(len(X))])
    
    matrix = [[sumX4, sumX3, sumX2, sumX2Y], [sumX3, sumX2, sum(X), sumXY], [sumX2, sum(X), len(X), sum(Y)]]
    
    GJ_method_abc = GJ_method_2(matrix)
    a = GJ_method_abc[0]
    b = GJ_method_abc[1]
    c = GJ_method_abc[2]
    
    x = Symbol('x')
    result = (eval('x')**2)*a + b*eval('x') + c
    
    return(result)

def newton1_function(X,Y):
    h=X[1]-X[0]
    
    # Найдем конечные разности
    y = copy.deepcopy(Y)
    deltay = [y[0]]
    while len(y) != 1:
        y = [y[i]-y[i-1] for i in range(1,len(y))]
        deltay.append(y[0])

    result=deltay[0]
    
    x = Symbol('x')
    deltax = [eval('x')-X[0]]
    for i in range(1,len(deltay)-1):
        deltax.append(deltax[i-1]*(eval('x') - X[i]))

    for i in range(1,len(deltax)+1):
        deltay[i] /= h**(i) * factorial(i)
        result+=(deltay[i]*deltax[i-1])
    return result

def function(x,y,z,col):
    if col == 1:
        f = [eval(input("y' = "))]
    elif col == 2:
        f1 = eval(input("y' = "))
        f2 = eval(input("z' = "))
        f = [f1,f2]
    return f

def diff_my(y1, y0, dx):
    y = (y1 - y0) / dx 
    return y


# ## Задание 1

# #### Итерация ОДУ

# In[4]:


# Два в одном

def iteration():
    
    x = Symbol('x')
    y = Symbol('y')
    z = Symbol('z')
    
    system = input('Хотите ли вы ввести систему из двух уравнений? (Да/Нет):')
    
    if system.lower() == 'нет':
        func = function(x,y,z,1)
        z0 = '-'
        try:
            x0y0 = list(map(float,input('Введите начальные условия [x0,y0]: ').split()))
            x0 = x0y0[0]
            y0 = x0y0[1]
        except ValueError:
            print('Ошибка ввода. Попробуйте ещё раз.')
            return iteration()
        except IndexError:
            print('Введено слишком мало чисел. Попробуйте ещё раз.')
            return iteration()
        
        try:
            ab = list(map(int,input('Введите желаемый интервал [a,b]: ').split()))
            a = ab[0]
            b = ab[1]
        except ValueError:
            print('Ошибка ввода. Попробуйте ещё раз.')
            return iteration()
        except IndexError:
            print('Введено слишком мало чисел. Попробуйте ещё раз.')
            return iteration()
        
        if a > b:
            print(f'Ошибка в вводе интервала! ({a}!>{b})!!')
            return iteration()
    
    elif system.lower() == 'да':
        func = function(x,y,z,2)
        
        try:
            x0y0 = list(map(float,input('Введите начальные условия [x0,y0,z0]: ').split()))
            x0 = x0y0[0]
            y0 = x0y0[1]
            z0 = x0y0[2]
        except ValueError:
            print('Ошибка ввода. Попробуйте ещё раз.')
            return iteration()
        except IndexError:
            print('Введено слишком мало чисел. Попробуйте ещё раз.')
            return iteration()
    
        try:
            ab = list(map(int,input('Введите желаемый интервал [a,b]: ').split()))
            a = ab[0]
            b = ab[1]
        except ValueError:
            print('Ошибка ввода. Попробуйте ещё раз.')
            return iteration()
        except IndexError:
            print('Введено слишком мало чисел. Попробуйте ещё раз.')
            return iteration()
        if a > b:
            print(f'Ошибка в вводе интервала! ({a}!>{b})!!')
            return iteration()
        
    n = int(input('Введите количество точек (n): '))
    
    return(func,x0,y0,z0,a,b,n)

# Одно уравнение

def iteration_once():
    
    x = Symbol('x')
    y = Symbol('y')
    z = Symbol('z')
    
    func = function(x,y,z,1)
    try:
        x0y0 = list(map(float,input('Введите начальные условия [x0,y0]: ').split()))
        x0 = x0y0[0]
        y0 = x0y0[1]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return iteration_once()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return iteration_once()

    try:
        ab = list(map(int,input('Введите желаемый интервал [a,b]: ').split()))
        a = ab[0]
        b = ab[1]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return iteration_once()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return iteration_once()

    if a > b:
        print(f'Ошибка в вводе интервала! ({a}!>{b})!!')
        return iiteration_once()
        
    n = int(input('Введите количество точек (n): '))
    
    return(func,x0,y0,a,b,n)

# Система уравнений

def iteration_system():
    
    x = Symbol('x')
    y = Symbol('y')
    z = Symbol('z')

    func = function(x,y,z,2)

    try:
        x0y0 = list(map(float,input('Введите начальные условия [x0,y0,z0]: ').split()))
        x0 = x0y0[0]
        y0 = x0y0[1]
        z0 = x0y0[2]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return iteration_system()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return iteration_system()
    
    try:
        ab = list(map(int,input('Введите желаемый интервал [a,b]: ').split()))
        a = ab[0]
        b = ab[1]
    except ValueError:
        print('Ошибка ввода. Попробуйте ещё раз.')
        return iteration_system()
    except IndexError:
        print('Введено слишком мало чисел. Попробуйте ещё раз.')
        return iteration_system()
    if a > b:
        print(f'Ошибка в вводе интервала! ({a}!>{b})!!')
        return iteration_system()
        
    n = int(input('Введите количество точек (n): '))
    
    return(func,x0,y0,z0,a,b,n)


# #### Решение ОДУ методом Эйлера

# In[5]:


# Два в одном

def euler(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    if type(z0) == type(''):
        res = [[i, x[i], 0] for i in range(n)]
        res[0][2] = y0
        for i in range(1,n):
            res[i][0] = i
            res[i][1] = x[i]
            res[i][2] = res[i-1][2] + (h*func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2])]))
    
    else:
        res = [[i, x[i], 0, 0] for i in range(n)]
        res[0][2] = y0
        res[0][3] = z0
        for i in range(1,n):
            res[i][0] = i
            res[i][1] = x[i]
            res[i][2] = res[i-1][2] + (h*func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))
            res[i][3] = res[i-1][3] + (h*func[1].subs([(X,res[i-1][1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))
    return res

# Одно уравнение

def euler_once(func, x0, y0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    res = [[i, x[i], 0] for i in range(n)]
    res[0][2] = y0
    for i in range(1,n):
        res[i][0] = i
        res[i][1] = x[i]
        res[i][2] = res[i-1][2] + (h*func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2])]))
    return res

# Система уравнений

def euler_system(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    res = [[i, x[i], 0, 0] for i in range(n)]
    res[0][2] = y0
    res[0][3] = z0
    for i in range(1,n):
        res[i][0] = i
        res[i][1] = x[i]
        res[i][2] = res[i-1][2] + (h*func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))
        res[i][3] = res[i-1][3] + (h*func[1].subs([(X,res[i-1][1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))
    return res


# #### Решение ОДУ методом Эйлера-Коши

# In[6]:


# Два в одном

def eulercauchy(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    if type(z0) == type(''):
        _y = [0]*n
        _y[0] = y0
        for i in range(1,n):
            _y[i] = _y[i-1] + (h*func[0].subs([(X,x[i-1]),(Y,_y[i-1])]))
        res = [[i, x[i], 0] for i in range(n)]
        res[0][2] = y0
        for i in range(1,n):
            res[i][2] = res[i-1][2] + (h/2) * ((func[0].subs([(X,x[i-1]),(Y,res[i-1][2])]))+(func[0].subs([(X,x[i]),(Y,_y[i])])))
    
    else:
        _y = [0]*n
        _z = [0]*n
        _y[0] = y0
        _z[0] = z0
        for i in range(1,n):
            _y[i] = _y[i-1] + (h*func[0].subs([(X,x[i-1]),(Y,_y[i-1]),(Z,_z[i-1])]))
            _z[i] = _z[i-1] + (h*func[1].subs([(X,x[i-1]),(Y,_y[i-1]),(Z,_z[i-1])]))
        res = [[i, x[i], 0, 0] for i in range(n)]
        res[0][2] = y0
        res[0][3] = z0
        for i in range(1,n):
            res[i][2] = res[i-1][2] + (h/2) * ((func[0].subs([(X,x[i-1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))+(func[0].subs([(X,x[i]),(Y,_y[i]),(Z,_z[i])])))
            res[i][3] = res[i-1][3] + (h/2) * ((func[1].subs([(X,x[i-1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))+(func[1].subs([(X,x[i]),(Y,_y[i]),(Z,_z[i])])))
    return res

# Одно уравнение

def eulercauchy_once(func, x0, y0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    _y = [0]*n
    _y[0] = y0
    for i in range(1,n):
        _y[i] = _y[i-1] + (h*func[0].subs([(X,x[i-1]),(Y,_y[i-1])]))
    res = [[i, x[i], 0] for i in range(n)]
    res[0][2] = y0
    for i in range(1,n):
        res[i][2] = res[i-1][2] + (h/2) * ((func[0].subs([(X,x[i-1]),(Y,res[i-1][2])]))+(func[0].subs([(X,x[i]),(Y,_y[i])])))
    return res

# Система уравнений

def eulercauchy_system(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    _y = [0]*n
    _z = [0]*n
    _y[0] = y0
    _z[0] = z0
    for i in range(1,n):
        _y[i] = _y[i-1] + (h*func[0].subs([(X,x[i-1]),(Y,_y[i-1]),(Z,_z[i-1])]))
        _z[i] = _z[i-1] + (h*func[1].subs([(X,x[i-1]),(Y,_y[i-1]),(Z,_z[i-1])]))
    res = [[i, x[i], 0, 0] for i in range(n)]
    res[0][2] = y0
    res[0][3] = z0
    for i in range(1,n):
        res[i][2] = res[i-1][2] + (h/2) * ((func[0].subs([(X,x[i-1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))+(func[0].subs([(X,x[i]),(Y,_y[i]),(Z,_z[i])])))
        res[i][3] = res[i-1][3] + (h/2) * ((func[1].subs([(X,x[i-1]),(Y,res[i-1][2]),(Z,res[i-1][3])]))+(func[1].subs([(X,x[i]),(Y,_y[i]),(Z,_z[i])])))
    return res


# #### Решение ОДУ методом Рунге-Кутты

# In[7]:


# Два в одном

def rungekutta(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    if type(z0) == type(''):
        res = [[i, x[i], 0] for i in range(n)]
        res[0][2] = y0
        for i in range(1,n):
            k1 = h * func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2])])
            k2 = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1/2)])
            k3 = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2/2)])
            k4 = h * func[0].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3)])
            res[i][2] = res[i-1][2] + 1/6 * (k1 + 2*k2 + 2*k3 + k4)
    
    else:
        res = [[i, x[i], 0, 0] for i in range(n)]
        res[0][2] = y0
        res[0][3] = z0
        for i in range(1,n):
            k1y = h * func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2]), (Z,res[i-1][3])])
            k2y = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1y/2), (Z,(res[i-1][3])+k1y/2)])
            k3y = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2y/2), (Z,(res[i-1][3])+k2y/2)])
            k4y = h * func[0].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3y),(Z,(res[i-1][3])+k3y)])
            res[i][2] = res[i-1][2] + 1/6 * (k1y + 2*k2y + 2*k3y + k4y)
            
            k1z = h * func[1].subs([(X,res[i-1][1]),(Y,res[i-1][2]), (Z,res[i-1][3])])
            k2z = h * func[1].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1z/2), (Z,(res[i-1][3])+k1z/2)])
            k3z = h * func[1].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2z/2), (Z,(res[i-1][3])+k2z/2)])
            k4z = h * func[1].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3z),(Z,(res[i-1][3])+k3z)])
            res[i][3] = res[i-1][3] + 1/6 * (k1z + 2*k2z + 2*k3z + k4z)
    return res

# Одно уравнение

def rungekutta_once(func, x0, y0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    res = [[i, x[i], 0] for i in range(n)]
    res[0][2] = y0
    for i in range(1,n):
        k1 = h * func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2])])
        k2 = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1/2)])
        k3 = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2/2)])
        k4 = h * func[0].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3)])
        res[i][2] = res[i-1][2] + 1/6 * (k1 + 2*k2 + 2*k3 + k4)
    return res

# Система уравнений

def rungekutta_system(func, x0, y0, z0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    res = [[i, x[i], 0, 0] for i in range(n)]
    res[0][2] = y0
    res[0][3] = z0
    for i in range(1,n):
        k1y = h * func[0].subs([(X,res[i-1][1]),(Y,res[i-1][2]), (Z,res[i-1][3])])
        k2y = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1y/2), (Z,(res[i-1][3])+k1y/2)])
        k3y = h * func[0].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2y/2), (Z,(res[i-1][3])+k2y/2)])
        k4y = h * func[0].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3y),(Z,(res[i-1][3])+k3y)])
        res[i][2] = res[i-1][2] + 1/6 * (k1y + 2*k2y + 2*k3y + k4y)

        k1z = h * func[1].subs([(X,res[i-1][1]),(Y,res[i-1][2]), (Z,res[i-1][3])])
        k2z = h * func[1].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k1z/2), (Z,(res[i-1][3])+k1z/2)])
        k3z = h * func[1].subs([(X,(res[i-1][1])+h/2),(Y,(res[i-1][2])+k2z/2), (Z,(res[i-1][3])+k2z/2)])
        k4z = h * func[1].subs([(X,(res[i-1][1])+h),(Y,(res[i-1][2])+k3z),(Z,(res[i-1][3])+k3z)])
        res[i][3] = res[i-1][3] + 1/6 * (k1z + 2*k2z + 2*k3z + k4z)
    return res


# #### Решение через библиотеку (Работает только с одним уравнением)

# In[8]:


def odeint_scp(func, x0, y0, a, b, n):
    h = (b-a)/n
    x = np.arange(x0,x0+(b-a),h)
    X = Symbol('x')
    Y = Symbol('y')
    Z = Symbol('z')
    f = lambdify([X,Y],func[0])
    res = [[i, x[i], 0] for i in range(n)]
    y = odeint(f,y0,np.array(x))
    for i in range(n):
        res[i][2] = float(y[i])
    return res


# #### Ввод "два в одном"

# In[ ]:


func, x0, y0, z0, a, b, n = iteration()
print(func)
print(x0)
print(y0)
print(z0)
print(a)
print(b)
print(n)


# In[11]:


euler(func, x0, y0, z0, a, b, n)


# In[12]:


eulercauchy(func, x0, y0, z0, a, b, n)


# In[13]:


rungekutta(func, x0, y0, z0, a, b, n)


# #### Ввод одного уравнения

# In[27]:


func, x0, y0, a, b, n = iteration_once()
print(func)
print(x0)
print(y0)
print(a)
print(b)
print(n)


# In[28]:


euler_once(func, x0, y0, a, b, n)


# In[29]:


eulercauchy_once(func, x0, y0, a, b, n)


# In[30]:


rungekutta_once(func, x0, y0, a, b, n)


# In[31]:


odeint_scp(func, x0, y0, a, b, n)


# #### Ввод системы уравнений

# In[19]:


func, x0, y0, z0, a, b, n = iteration_system()
print(func)
print(x0)
print(y0)
print(a)
print(b)
print(n)


# In[20]:


euler_system(func, x0, y0, z0, a, b, n)


# In[21]:


eulercauchy_system(func, x0, y0, z0, a, b, n)


# In[22]:


rungekutta_system(func, x0, y0, z0, a, b, n)


# ## Задание 2

# #### Ввод "два в одном"

# In[11]:


func, x0, y0, z0, a, b, n = iteration()

t0 = time.time()
result_euler = euler(func, x0, y0, z0, a, b, n)
t1_euler = time.time()
t1_euler -= t0

t0 = time.time()
result_eulercauchy = eulercauchy(func, x0, y0, z0, a, b, n)
t1_eulercauchy = time.time()
t1_eulercauchy -= t0

t0 = time.time()
result_rungekutta = rungekutta(func, x0, y0, z0, a, b, n)
t1_rungekutta = time.time()
t1_rungekutta -= t0

xd = [result_euler[i][1] for i in range(n)]
yd_euler = [result_euler[i][2] for i in range(n)]
yd_eulercauchy = [result_eulercauchy[i][2] for i in range(n)]
yd_rungekutta = [result_rungekutta[i][2] for i in range(n)]

# Эйлер
if len(result_euler[0]) == 3:
    df_euler = pd.DataFrame({'xi':xd,'yi':yd_euler})
elif len(result_euler[0]) == 4:
    zd_euler = [result_euler[i][3] for i in range(n)]
    df_euler = pd.DataFrame({'xi':xd,'yi':yd_euler,'zi':zd_euler})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера') 
print(df_euler)

# Эйлер-Коши
if len(result_eulercauchy[0]) == 3:
    df_eulercauchy = pd.DataFrame({'xi':xd,'yi':yd_eulercauchy})
elif len(result_eulercauchy[0]) == 4:
    zd_eulercauchy = [result_eulercauchy[i][3] for i in range(n)]
    df_eulercauchy = pd.DataFrame({'xi':xd,'yi':yd_eulercauchy,'zi':zd_eulercauchy})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера-Коши') 
print(df_eulercauchy)

# Рунге-Кутта
if len(result_rungekutta[0]) == 3:
    df_rungekutta = pd.DataFrame({'xi':xd,'yi':yd_rungekutta})
elif len(result_rungekutta[0]) == 4:
    zd_rungekutta = [result_rungekutta[i][3] for i in range(n)]
    df_rungekutta = pd.DataFrame({'xi':xd,'yi':yd_rungekutta,'zi':zd_rungekutta})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Рунге-Кутты') 
print(df_rungekutta)

# scipy.integrate odeint
if type(z0) == type(''):
    t0 = time.time()
    result_odeint_scp = odeint_scp(func, x0, y0, a, b, n)
    t1_odeint_scp = time.time()
    t1_odeint_scp -= t0
    
    yd_odeint_scp = [result_odeint_scp[i][2] for i in range(n)]
    df_odeint_scp = pd.DataFrame({'xi':xd,'yi':yd_odeint_scp})
    
    
    print('--------------------------------------------------------------')
    print('Решение ОДУ scipy.integrate odeint') 
    print(df_odeint_scp)

dxa = xd[1] - xd[0]
ymydiff = []
for i in range(len(yd_odeint_scp) - 1):
    ansi = diff_my(yd_odeint_scp[1],yd_odeint_scp[0],dxa)
    ymydiff.append(ansi)

df_ymydiff = pd.DataFrame({'xi':xd,'deltai':yd_odeint_scp})
print('--------------------------------------------------------------')
print('Разность между yi и y' + "'" + 'i') 
print(df_ymydiff)       
    
deltai = [(yd_odeint_scp[i] - ymydiff[i]) for i in range(len(ymydiff))]
sumx = sum([abs(t) for t in deltai])
rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300
fig = plt.figure()
fig,ax = plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
ax.plot(xd[:-1],deltai,color='blue',lw=2,label='Y - Y' + "'")
ax.plot(xd[:-1],deltai,color='blue',lw=0,label=f'Сумма отклонений = {sumx}')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=20)
plt.show()


# In[24]:


from mpl_toolkits.mplot3d import Axes3D

rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

if len(result_euler[0]) == 3:

    fig,ax=plt.subplots()

    plt.tick_params(labelsize = 40)
    plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

    ax.plot(xd,yd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
    ax.plot(xd,yd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
    ax.plot(xd,yd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')
    ax.plot(xd,yd_odeint_scp,color='black',lw=2,label='Решение ОДУ scipy.integrate odeint')

    ax.set_title('Решение ОДУ')
    ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

    plt.xlabel('X',fontsize=25)
    plt.ylabel('Y',fontsize=25)
    
    print(f'Аппроксимация МНК для метода Эйлера (XY): \n {quadratic_function(xd,yd_euler)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XY): \n {expand(newton1_function(xd,yd_euler))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Эйлера-Коши (XY): \n {quadratic_function(xd,yd_eulercauchy)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XY): \n {expand(newton1_function(xd,yd_eulercauchy))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_rungekutta)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_rungekutta))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_odeint_scp)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_odeint_scp))}')
    
else:
    fig = plt.figure()

    axes = fig.subplots(nrows=2, ncols=1)

    axes[0].plot(xd,yd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
    axes[0].plot(xd,yd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
    axes[0].plot(xd,yd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')
    axes[0].grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
    axes[0].set_title('XY',fontfamily = 'fantasy', loc = 'left')
    axes[0].set_xlabel('X')
    axes[0].set_ylabel('Y')

    axes[1].plot(xd,zd_euler,color='blue',lw=2)
    axes[1].plot(xd,zd_eulercauchy,color='red',lw=2)
    axes[1].plot(xd,zd_rungekutta,color='green',lw=2)
    axes[1].grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
    axes[1].set_title('XZ',fontfamily = 'fantasy',loc = 'left')
    axes[1].set_xlabel('X')
    axes[1].set_ylabel('Z')

    lines = []
    labels = []

    for ax in fig.axes:
        axLine, axLabel = ax.get_legend_handles_labels()
        lines.extend(axLine)
        labels.extend(axLabel)

    fig.legend(lines, labels,           
               loc = 'upper right',
              fontsize=30)
    
    # 3d
    
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    plt.tick_params(labelsize = 40)
    plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

    ax.plot(xd,yd_euler,zd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
    ax.plot(xd,yd_eulercauchy,zd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
    ax.plot(xd,yd_rungekutta,zd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')

    ax.set_title('Решение ОДУ')
    ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

    plt.xlabel('X',fontsize=25)
    plt.ylabel('Y',fontsize=25)
    
    print(f'Аппроксимация МНК для метода Эйлера (XY): \n {quadratic_function(xd,yd_euler)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XY): \n {expand(newton1_function(xd,yd_euler))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Эйлера-Коши (XY): \n {quadratic_function(xd,yd_eulercauchy)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XY): \n {expand(newton1_function(xd,yd_eulercauchy))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_rungekutta)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_rungekutta))}')
    print('-----------------------------------------------------------------------------------')

    print(f'Аппроксимация МНК для метода Эйлера (XZ): \n {quadratic_function(xd,zd_euler)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XZ): \n {expand(newton1_function(xd,zd_euler))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Эйлера-Коши (XZ): \n {quadratic_function(xd,zd_eulercauchy)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XZ): \n {expand(newton1_function(xd,zd_eulercauchy))}')
    print('-----------------------------------------------------------------------------------')
    print(f'Аппроксимация МНК для метода Рунге-Кутты (XZ): \n {quadratic_function(xd,zd_rungekutta)}')
    print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XZ): \n {expand(newton1_function(xd,zd_rungekutta))}')
    
plt.show()

t=[]
print(f'Время работы метода Эйлера: {t1_euler}.')
t.append(t1_euler)
print(f'Время работы метода Эйлера-Коши: {t1_eulercauchy}.')
t.append(t1_eulercauchy)
print(f'Время работы метода Рунге-Кутты: {t1_rungekutta}.')
t.append(t1_rungekutta)
if type(z0) == type(''):
    print(f'Время работы метода из библиотеки: {t1_odeint_scp}.')
    t.append(t1_odeint_scp)


# In[34]:


rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

fig,ax=plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

plt.bar(0, t[0], color = 'purple', label = 'Время работы метода Эйлера') 
plt.bar(1, t[1], color = 'yellow', label = 'Время работы метода Эйлера-Коши') 
plt.bar(2, t[2], color = 'lime', label = 'Время работы метода Рунге-Кутты')
if len(t) == 4:
    plt.bar(3, t[3], color = 'red', label = 'Время работы метода из библиотеки') 

ax.set_title('Анализ времени работы программ', loc = 'center')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

plt.ylabel('Время (cек.)',fontsize=25)

plt.show()


# #### Ввод одного уравнения

# In[12]:


func, x0, y0, a, b, n = iteration_once()

t0 = time.time()
result_euler = euler_once(func, x0, y0, a, b, n)
t1_euler = time.time()
t1_euler -= t0

t0 = time.time()
result_eulercauchy = eulercauchy_once(func, x0, y0, a, b, n)
t1_eulercauchy = time.time()
t1_eulercauchy -= t0

t0 = time.time()
result_rungekutta = rungekutta_once(func, x0, y0, a, b, n)
t1_rungekutta = time.time()
t1_rungekutta -= t0

t0 = time.time()
result_odeint_scp = odeint_scp(func, x0, y0, a, b, n)
t1_odeint_scp = time.time()
t1_odeint_scp -= t0

xd = [result_euler[i][1] for i in range(n)]
yd_euler = [result_euler[i][2] for i in range(n)]
yd_eulercauchy = [result_eulercauchy[i][2] for i in range(n)]
yd_rungekutta = [result_rungekutta[i][2] for i in range(n)]
yd_odeint_scp = [result_odeint_scp[i][2] for i in range(n)]

# Эйлер
df_euler = pd.DataFrame({'xi':xd,'yi':yd_euler})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера') 
print(df_euler)

# Эйлер-Коши
df_eulercauchy = pd.DataFrame({'xi':xd,'yi':yd_eulercauchy})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера-Коши') 
print(df_eulercauchy)

# Рунге-Кутта
df_rungekutta = pd.DataFrame({'xi':xd,'yi':yd_rungekutta})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Рунге-Кутты') 
print(df_rungekutta)

# scipy.integrate odeint
df_odeint_scp = pd.DataFrame({'xi':xd,'yi':yd_odeint_scp})
    
print('--------------------------------------------------------------')
print('Решение ОДУ scipy.integrate odeint') 
print(df_odeint_scp)

dxa = xd[1] - xd[0]
ymydiff = []
for i in range(len(yd_odeint_scp) - 1):
    ansi = diff_my(yd_odeint_scp[1],yd_odeint_scp[0],dxa)
    ymydiff.append(ansi)

df_ymydiff = pd.DataFrame({'xi':xd,'deltai':yd_odeint_scp})
print('--------------------------------------------------------------')
print('Разность между yi и y' + "'" + 'i') 
print(df_ymydiff)       
    
deltai = [(yd_odeint_scp[i] - ymydiff[i]) for i in range(len(ymydiff))]
sumx = sum([abs(t) for t in deltai])
rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300
fig = plt.figure()
fig,ax = plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
ax.plot(xd[:-1],deltai,color='blue',lw=2,label='Y - Y' + "'")
ax.plot(xd[:-1],deltai,color='blue',lw=0,label=f'Сумма отклонений = {sumx}')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=20)
plt.show()


# In[36]:


rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

fig,ax=plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

ax.plot(xd,yd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
ax.plot(xd,yd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
ax.plot(xd,yd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')
ax.plot(xd,yd_odeint_scp,color='black',lw=2,label='Решение ОДУ scipy.integrate odeint')

ax.set_title('Решение ОДУ')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)

print(f'Аппроксимация МНК для метода Эйлера (XY): \n {quadratic_function(xd,yd_euler)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XY): \n {expand(newton1_function(xd,yd_euler))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Эйлера-Коши (XY): \n {quadratic_function(xd,yd_eulercauchy)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XY): \n {expand(newton1_function(xd,yd_eulercauchy))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_rungekutta)}')
print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_rungekutta))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_odeint_scp)}')
print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_odeint_scp))}')

t=[]
print(f'Время работы метода Эйлера: {t1_euler}.')
t.append(t1_euler)
print(f'Время работы метода Эйлера-Коши: {t1_eulercauchy}.')
t.append(t1_eulercauchy)
print(f'Время работы метода Рунге-Кутты: {t1_rungekutta}.')
t.append(t1_rungekutta)
print(f'Время работы метода из библиотеки: {t1_odeint_scp}.')
t.append(t1_odeint_scp)


# In[14]:


rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

fig,ax=plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

plt.bar(0, t[0], color = 'purple', label = 'Время работы метода Эйлера') 
plt.bar(1, t[1], color = 'yellow', label = 'Время работы метода Эйлера-Коши') 
plt.bar(2, t[2], color = 'lime', label = 'Время работы метода Рунге-Кутты')
plt.bar(3, t[3], color = 'red', label = 'Время работы метода из библиотеки') 

ax.set_title('Анализ времени работы программ', loc = 'center')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

plt.ylabel('Время (cек.)',fontsize=25)

plt.show()


# #### Ввод системы уравнений

# In[15]:


func, x0, y0, z0, a, b, n = iteration_system()

t0 = time.time()
result_euler = euler_system(func, x0, y0, z0, a, b, n)
t1_euler = time.time()
t1_euler -= t0

t0 = time.time()
result_eulercauchy = eulercauchy_system(func, x0, y0, z0, a, b, n)
t1_eulercauchy = time.time()
t1_eulercauchy -= t0

t0 = time.time()
result_rungekutta = rungekutta_system(func, x0, y0, z0, a, b, n)
t1_rungekutta = time.time()
t1_rungekutta -= t0

xd = [result_euler[i][1] for i in range(n)]
yd_euler = [result_euler[i][2] for i in range(n)]
yd_eulercauchy = [result_eulercauchy[i][2] for i in range(n)]
yd_rungekutta = [result_rungekutta[i][2] for i in range(n)]

# Эйлер
zd_euler = [result_euler[i][3] for i in range(n)]
df_euler = pd.DataFrame({'xi':xd,'yi':yd_euler,'zi':zd_euler})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера') 
print(df_euler)

# Эйлер-Коши
zd_eulercauchy = [result_eulercauchy[i][3] for i in range(n)]
df_eulercauchy = pd.DataFrame({'xi':xd,'yi':yd_eulercauchy,'zi':zd_eulercauchy})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Эйлера-Коши') 
print(df_eulercauchy)

# Рунге-Кутта
zd_rungekutta = [result_rungekutta[i][3] for i in range(n)]
df_rungekutta = pd.DataFrame({'xi':xd,'yi':yd_rungekutta,'zi':zd_rungekutta})
    
print('--------------------------------------------------------------')
print('Решение ОДУ методом Рунге-Кутты') 
print(df_rungekutta)


# In[16]:


fig = plt.figure()

rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

axes = fig.subplots(nrows=2, ncols=1)

axes[0].plot(xd,yd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
axes[0].plot(xd,yd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
axes[0].plot(xd,yd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')
axes[0].grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
axes[0].set_title('XY',fontfamily = 'fantasy', loc = 'left')
axes[0].set_xlabel('X')
axes[0].set_ylabel('Y')

axes[1].plot(xd,zd_euler,color='blue',lw=2)
axes[1].plot(xd,zd_eulercauchy,color='red',lw=2)
axes[1].plot(xd,zd_rungekutta,color='green',lw=2)
axes[1].grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)
axes[1].set_title('XZ',fontfamily = 'fantasy',loc = 'left')
axes[1].set_xlabel('X')
axes[1].set_ylabel('Z')

lines = []
labels = []

for ax in fig.axes:
    axLine, axLabel = ax.get_legend_handles_labels()
    lines.extend(axLine)
    labels.extend(axLabel)

fig.legend(lines, labels,           
           loc = 'upper right',
          fontsize=30)

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

ax.plot(xd,yd_euler,zd_euler,color='blue',lw=2,label='Решение ОДУ методом Эйлера')
ax.plot(xd,yd_eulercauchy,zd_eulercauchy,color='red',lw=2,label='Решение ОДУ методом Эйлера-Коши')
ax.plot(xd,yd_rungekutta,zd_rungekutta,color='green',lw=2,label='Решение ОДУ методом Рунге-Кутты')

ax.set_title('Решение ОДУ')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

plt.xlabel('X',fontsize=25)
plt.ylabel('Y',fontsize=25)

print(f'Аппроксимация МНК для метода Эйлера (XY): \n {quadratic_function(xd,yd_euler)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XY): \n {expand(newton1_function(xd,yd_euler))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Эйлера-Коши (XY): \n {quadratic_function(xd,yd_eulercauchy)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XY): \n {expand(newton1_function(xd,yd_eulercauchy))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Рунге-Кутты (XY): \n {quadratic_function(xd,yd_rungekutta)}')
print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XY): \n {expand(newton1_function(xd,yd_rungekutta))}')
print('-----------------------------------------------------------------------------------')

print(f'Аппроксимация МНК для метода Эйлера (XZ): \n {quadratic_function(xd,zd_euler)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера (XZ): \n {expand(newton1_function(xd,zd_euler))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Эйлера-Коши (XZ): \n {quadratic_function(xd,zd_eulercauchy)}')
print(f'Интерполяция через первую формулу Ньютона для метода Эйлера-Коши (XZ): \n {expand(newton1_function(xd,zd_eulercauchy))}')
print('-----------------------------------------------------------------------------------')
print(f'Аппроксимация МНК для метода Рунге-Кутты (XZ): \n {quadratic_function(xd,zd_rungekutta)}')
print(f'Интерполяция через первую формулу Ньютона для метода Рунге-Кутты (XZ): \n {expand(newton1_function(xd,zd_rungekutta))}')

plt.show()

t=[]
print(f'Время работы метода Эйлера: {t1_euler}.')
t.append(t1_euler)
print(f'Время работы метода Эйлера-Коши: {t1_eulercauchy}.')
t.append(t1_eulercauchy)
print(f'Время работы метода Рунге-Кутты: {t1_rungekutta}.')
t.append(t1_rungekutta)


# In[17]:


rcParams['figure.figsize'] = (30, 20)
rcParams['figure.dpi'] = 300

fig,ax=plt.subplots()

plt.tick_params(labelsize = 40)
plt.grid(b=True, color='DarkTurquoise', alpha=0.75, linestyle=':', linewidth=1)

plt.bar(0, t[0], color = 'purple', label = 'Время работы метода Эйлера') 
plt.bar(1, t[1], color = 'yellow', label = 'Время работы метода Эйлера-Коши') 
plt.bar(2, t[2], color = 'lime', label = 'Время работы метода Рунге-Кутты')

ax.set_title('Анализ времени работы программ', loc = 'center')
ax.legend(loc='lower right',title='Легенда',title_fontsize=25,fontsize=15)

plt.ylabel('Время (cек.)',fontsize=25)

plt.show()


# In[ ]:







#!/usr/bin/env python

from io import open
from setuptools import setup

"""
:authors: Runner
:license: Apache License, Version 2.0, see LICENSE file
:copyright: (c) 2021 Peopl3s
"""

version = '1.0.1'

long_description = ""

setup(
    name='hw_bonus_point_api',
    version=version,

    author='Runner',
    author_email='levintsov.v.v@gmail.com',

    description=(
        u'Python module for writing scripts for project management platform '
        u'Club House (clubhouse.io API wrapper)'
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/EgorMozharov/hw_bonus_point',
    download_url='https://github.com/EgorMozharov/hw_bonus_point/archive/v{}.zip'.format(version),

    license='Apache License, Version 2.0, see LICENSE file',

    packages=['hw_bonus_point_api'],
    install_requires=['aiohttp', 'aiofiles'],

    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
)
